import dayjs from 'dayjs';
import type { DateType, CalendarDay, CalendarMonth, CalendarWeek, Numerals, CalendarType } from './types';
import { type ClassValue } from 'clsx';
export declare const CALENDAR_FORMAT = "YYYY-MM-DD HH:mm";
export declare const DATE_FORMAT = "YYYY-MM-DD";
export declare const YEAR_PAGE_SIZE = 12;
export declare const VALID_JALALI_LOCALES: Set<string>;
export declare const JALALI_MONTHS: {
    en: string[];
    fa: string[];
};
export declare const isValidJalaliLocale: (locale: string) => boolean;
export declare const getJalaliMonths: (locale: string) => string[];
export declare const getMonths: () => dayjs.MonthNames;
export declare const getMonthName: (month: number) => string | undefined;
/**
 * Converts a date to a dayjs object with optional Islamic calendar support.
 *
 * @param {DateType} date - The input date to convert (can be any value dayjs accepts)
 * @param {CalendarType} [calendar] - Optional calendar type to use ('islamic' for Hijri calendar)
 * @returns {Dayjs} A dayjs object in either the default or Islamic calendar system
 *
 * @example
 * // Get a standard dayjs object in the Geogorian calendar
 * const standard = getDayjs('2025-03-20');
 *
 * @example
 * // Get a dayjs object of a date in the Hijri calendar
 * const hijri = getDayjs('2025-03-20', 'islamic');
 */
export declare const getDayjs: (date: DateType, calendar?: CalendarType) => dayjs.Dayjs;
/**
 * Get months array
 *
 * @returns months array
 */
export declare const getMonthsArray: ({ calendar, locale, }: {
    calendar: CalendarType;
    locale: string;
}) => CalendarMonth[];
/**
 * Get weekdays
 *
 * @param locale - locale
 * @param firstDayOfWeek - first day of week
 * @param format - format short, min or full
 *
 * @returns weekdays
 */
export declare const getWeekdays: (locale: string, firstDayOfWeek: number) => CalendarWeek[];
export declare const getFormated: (date: DateType, calendar?: CalendarType) => string;
export declare const getDateMonth: (date: DateType, calendar?: CalendarType) => number;
export declare const getDateYear: (date: DateType, calendar?: CalendarType) => number;
/**
 * Check if two dates are on the same day
 *
 * @param a - date to check
 * @param b - date to check
 * @param {CalendarType} [calendar] - Optional calendar type to use ('islamic' for Hijri calendar)
 *
 * @returns true if dates are on the same day, false otherwise
 */
export declare function areDatesOnSameDay(a: DateType, b: DateType, calendar?: CalendarType): boolean;
/**
 * Check if date is between two dates
 *
 * @param date - date to check
 * @param options - options
 * @param {CalendarType} [calendar] - Optional calendar type to use ('islamic' for Hijri calendar)
 *
 * @returns true if date is between two dates, false otherwise
 */
export declare function isDateBetween(date: DateType, { startDate, endDate, }: {
    startDate?: DateType;
    endDate?: DateType;
}, calendar?: CalendarType): boolean;
/**
 * Check if date is disabled
 *
 * @param date - date to check
 * @param options - options
 *
 * @returns true if date is disabled, false otherwise
 */
export declare function isDateDisabled(date: dayjs.Dayjs, { minDate, maxDate, enabledDates, disabledDates, calendar, }: {
    minDate?: DateType;
    maxDate?: DateType;
    enabledDates?: DateType[] | ((date: DateType) => boolean) | undefined;
    disabledDates?: DateType[] | ((date: DateType) => boolean) | undefined;
    calendar?: CalendarType;
}): boolean;
/**
 * Check if year is disabled
 *
 * @param year - year to check
 * @param options - options
 * @param {CalendarType} [calendar] - Optional calendar type to use ('islamic' for Hijri calendar)
 *
 * @returns true if year is disabled, false otherwise
 */
export declare function isYearDisabled(year: number, { minDate, maxDate, }: {
    minDate?: DateType;
    maxDate?: DateType;
}, calendar?: CalendarType): boolean;
/**
 * Check if month is disabled
 *
 * @param month - month to check
 * @param date - date to check
 * @param options - options
 * @param {CalendarType} [calendar] - Optional calendar type to use ('islamic' for Hijri calendar)
 *
 * @returns true if month is disabled, false otherwise
 */
export declare function isMonthDisabled(month: number, date: DateType, { minDate, maxDate, }: {
    minDate?: DateType;
    maxDate?: DateType;
}, calendar?: CalendarType): boolean;
/**
 * Get formated date
 *
 * @param date - date to get formated date from
 * @param format - format to get formated date from
 * @param {CalendarType} [calendar] - Optional calendar type to use ('islamic' for Hijri calendar)
 *
 * @returns formated date
 */
export declare const getFormatedDate: (date: DateType, format: string, calendar?: CalendarType) => string;
/**
 * Get date
 *
 * @param date - date to get
 * @param {CalendarType} [calendar] - Optional calendar type to use ('islamic' for Hijri calendar)
 *
 * @returns date
 */
export declare const getDate: (date: DateType, calendar?: CalendarType) => dayjs.Dayjs;
/**
 * Get year range
 *
 * @param year - year to get year range from
 *
 * @returns year range
 */
export declare const getYearRange: (year: number) => number[];
/**
 * Determines the number of days in a specific month of the Islamic (Hijri) calendar.
 *
 * This function calculates the number of days in a given Hijri month by checking
 * the last day's representation in the Islamic calendar. Most Hijri months are
 * either 29 or 30 days long.
 *
 * @param {dayjs.Dayjs} date - A dayjs object representing a reference date in the Islamic calendar
 * @returns {number} The number of days in the specified Hijri month (29 or 30)
 *
 * @example
 * // Determine days in a specific Hijri month
 * const specificMonth = getDaysInHijriMonth(dayjs('1446-04-01').toCalendarSystem('islamic'));
 *
 * @note
 * - The function uses the Umm al-Qura calendar system
 * - Month indexing starts at 1 (1 = Muharram, 12 = Dhu al-Hijjah) so +1 is added
 * - Relies on @umalqura/core calendar conversion
 */
export declare function getDaysInHijriMonth(date: dayjs.Dayjs): number;
/**
 * Get days in month
 *
 * @param date - date to get days in month from
 * @param showOutsideDays - whether to show outside days
 * @param firstDayOfWeek - first day of week, number 0-6, 0 – Sunday, 6 – Saturday
 * @param {CalendarType} [calendar] - Optional calendar type to use ('islamic' for Hijri calendar)
 *
 * @returns days in month
 */
export declare function getDaysInMonth(date: DateType, showOutsideDays: boolean | undefined, firstDayOfWeek: number, calendar?: CalendarType): {
    prevMonthDays: number;
    prevMonthOffset: number;
    daysInCurrentMonth: number;
    daysInNextMonth: number;
    fullDaysInMonth: number;
};
/**
 * Get first day of month
 *
 * @param date - date to get first day of month from
 * @param firstDayOfWeek - first day of week, number 0-6, 0 – Sunday, 6 – Saturday
 * @param {CalendarType} [calendar] - Optional calendar type to use ('islamic' for Hijri calendar)
 *
 * @returns first day of month
 */
export declare function getFirstDayOfMonth(date: DateType, firstDayOfWeek: number, calendar?: CalendarType): number;
/**
 * Get start of day
 *
 * @param date - date to get start of day from
 * @param {CalendarType} [calendar] - Optional calendar type to use ('islamic' for Hijri calendar)
 *
 * @returns start of day
 */
export declare function getStartOfDay(date: DateType, calendar?: CalendarType): DateType;
/**
 * Get end of day
 *
 * @param date - date to get end of day from
 * @param {CalendarType} [calendar] - Optional calendar type to use ('islamic' for Hijri calendar)
 *
 * @returns end of day
 */
export declare function getEndOfDay(date: DateType, calendar?: CalendarType): DateType;
/**
 * Convert date to unix timestamp
 *
 * @param date - date to convert
 * @param {CalendarType} [calendar] - Optional calendar type to use ('islamic' for Hijri calendar)
 *
 * @returns unix timestamp
 */
export declare function dateToUnix(date: DateType, calendar?: CalendarType): number;
/**
 * Remove time from date
 *
 * @param date - date to remove time from
 * @param {CalendarType} [calendar] - Optional calendar type to use ('islamic' for Hijri calendar)
 *
 * @returns date with time removed
 */
export declare function removeTime(date: DateType, timeZone: string | undefined, calendar?: CalendarType): DateType;
/**
 * Get detailed date object
 *
 * @param date Get detailed date object
 * @param {CalendarType} [calendar] - Optional calendar type to use ('islamic' for Hijri calendar)
 *
 * @returns parsed date object
 */
export declare const getParsedDate: (date: DateType, calendar?: CalendarType) => {
    year: number;
    month: number;
    hour: number;
    hour12: number;
    minute: number;
    period: string;
};
/**
 * Calculate month days array based on current date
 *
 * @param datetime - The current date that selected
 * @param showOutsideDays
 * @param minDate - min selectable date
 * @param maxDate - max selectable date
 * @param firstDayOfWeek - first day of week, number 0-6, 0 – Sunday, 6 – Saturday
 * @param enabledDates - array of enabled dates, or a function that returns true for a given date (takes precedence over disabledDates)
 * @param disabledDates - array of disabled dates, or a function that returns true for a given date
 * @param prevMonthDays - number of days in the previous month
 * @param prevMonthOffset - number of days to offset the previous month
 * @param daysInCurrentMonth - number of days in the current month
 * @param daysInNextMonth - number of days in the next month
 * @param {CalendarType} [calendar] - Optional calendar type to use ('islamic' for Hijri calendar)
 *
 * @returns days array based on current date
 */
export declare const getMonthDays: (datetime: DateType, showOutsideDays: boolean, minDate: DateType, maxDate: DateType, firstDayOfWeek: number, enabledDates: DateType[] | ((date: DateType) => boolean) | undefined, disabledDates: DateType[] | ((date: DateType) => boolean) | undefined, prevMonthDays: number, prevMonthOffset: number, daysInCurrentMonth: number, daysInNextMonth: number, numerals: Numerals, calendar?: CalendarType) => CalendarDay[];
export declare function cn(...inputs: ClassValue[]): string;
/**
 * Deep compare memo
 *
 * @param value - value to compare
 * @param deps - dependencies
 *
 * @returns memoized value
 */
export declare function useDeepCompareMemo<T>(value: T, deps: any[]): T;
export declare function formatNumber(value: number, numerals: Numerals): string;
//# sourceMappingURL=utils.d.ts.map